from ._core import Enum

class GroupEventType(Enum):
	"""Used to specify which type of Zalo group event is being used."""
	
	JOIN = "join"
	LEAVE = "leave"
	UPDATE = "update"
	UNKNOWN = "unknown"
	REACTION = "reaction"
	NEW_LINK = "new_link"
	ADD_ADMIN = "add_admin"
	REMOVE_ADMIN = "remove_admin"
	JOIN_REQUEST = "join_request"
	BLOCK_MEMBER = "block_member"
	REMOVE_MEMBER = "remove_member"
	UPDATE_SETTING = "update_setting"


class EventType(Enum):
	"""Used to specify which type of Zalo user event is being used."""
	
	REACTION = "reaction"
	


# -*- coding: UTF-8 -*-
import attr
import random
import requests, json

from . import _util, _exception

headers = {
	"User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36",
	"Accept": "application/json, text/plain, */*",
	"sec-ch-ua": "\"Not-A.Brand\";v=\"99\", \"Chromium\";v=\"124\"",
	"sec-ch-ua-mobile": "?0",
	"sec-ch-ua-platform": "\"Linux\"",
	"origin": "https://chat.zalo.me",
	"sec-fetch-site": "same-site",
	"sec-fetch-mode": "cors",
	"sec-fetch-dest": "empty",
	"Accept-Encoding": "gzip",
	"referer": "https://chat.zalo.me/",
	"accept-language": "vi-VN,vi;q=0.9,fr-FR;q=0.8,fr;q=0.7,en-US;q=0.6,en;q=0.5",
}
class State(object):
	def __init__(cls):
		cls._config = {}
		cls._headers = _util.HEADERS
		cls._cookies = _util.COOKIES
		cls._session = requests.Session()
		cls.user_id = None
		cls.user_imei = None
		cls._loggedin = False
	
	def get_cookies(cls):
		return cls._cookies
	
	def set_cookies(cls, cookies):
		cls._cookies = cookies
		
	def get_secret_key(cls):
		return cls._config.get("secret_key")
	
	def set_secret_key(cls, secret_key):
		cls._config["secret_key"] = secret_key
	
	def _get(cls, *args, **kwargs):
		sessionObj = cls._session.get(*args, **kwargs, headers=cls._headers, cookies=cls._cookies)
		
		return sessionObj
		
	def _post(cls, *args, **kwargs):
		sessionObj = cls._session.post(*args, **kwargs, headers=cls._headers, cookies=cls._cookies)
		return sessionObj
	
	def is_logged_in(cls):
		return cls._loggedin
	
	def login(cls, phone, password, imei, session_cookies=None, user_agent=None):
		if cls._cookies and cls._config.get("secret_key"):
			cls._loggedin = True
			return
			
		if user_agent:
			cls._headers["User-Agent"] = user_agent
			
		if cls._cookies:
			params = {
				"imei": imei,
			}
			try:
				url = f"https://wpa.chat.zalo.me/api/login/getLoginInfo?imei={imei}&type=30&client_version=645&computer_name=Web&ts={_util.now()}"
				response = requests.get(url, headers=headers, cookies=cls._cookies)
				data = response.json()
				zpw = data["data"]["zpw_ws"]
				uid = data["data"]["uid"]
				phone = data["data"]["phone_number"]
				key = data["data"]["zpw_enk"]
				
				content = {
					"data": {
						"phone_number": str(phone),
						"secret_key": str(key),
						"send2me_id": str(uid),
						"zpw_ws": zpw,
					},
				"error_code": 0
				}
				
				if content.get("error_code") == 0:
					cls._config = content.get("data")
					
					if cls._config.get("secret_key"):
						cls._loggedin = True
						cls.user_id = cls._config.get("send2me_id")
						cls.user_imei = imei
						
					else:
						cls._loggedin = False
						raise _exception.ZaloLoginError("Unable to get `secret key`.")
						
				else:
					error = data.get("error_code")
					content = data.get("error_message")
					raise _exception.ZaloLoginError(f"Error #{error} when logging in: {content}")
			
			except _exception.ZaloLoginError as e:
				raise _exception.ZaloLoginError(str(e))
				
			except Exception as e:
				raise _exception.ZaloLoginError(f"An error occurred while logging in! {str(e)}")
		
		else:
			raise _exception.LoginMethodNotSupport("Login method is not supported yet")
		
	



# -*- coding: UTF-8 -*-
import traceback

from ._util import now, urllib
from ._core import Enum
from ._exception import (
	ZaloAPIException,
	ZaloUserError,
	ZaloLoginError,
	LoginMethodNotSupport,
	EncodePayloadError,
	DecodePayloadError
)
from ._threads import ThreadType
from ._aevents import GroupEventType, EventType
from ._message import MessageReaction, MessageStyle, MultiMsgStyle, Message, Mention, MultiMention
from ._objects import User, Group, MessageObject, ContextObject, EventObject

from .logging import Logging

logger = Logging(theme="catppuccin-mocha", log_text_color="black")


# -*- coding: UTF-8 -*-
"""Zalo API for Python

:copyright: (c) 2024 by Lê Quốc Việt (Vexx).
"""
from .models import *
from ._client import ZaloAPI

__title__ = "zlapi"
__version__ = "1.0.2"
__description__ = "Zalo API (Website) for Python"

__copyright__ = "Copyright 2024 by Vexx"

__author__ = "Lê Quốc Việt"
__email__ = "vrxxdev@gmail.com"

__all__ = ["ZaloAPI"]



class ZaloAPIException(Exception):
    """Custom exception thrown by ``zaoapi``.

    All exceptions in the ``zaloapi`` module inherits this.
    """

class LoginMethodNotSupport(ZaloAPIException):
	"""Raised by zaloapi if:

    - Using an unsupported login method.
    """
	def __init__(self, message=None):
		self.message = message
		super().__init__(message)
		

class ZaloLoginError(ZaloAPIException):
	def __init__(self, message=None):
		self.message = message
		super().__init__(message)
		

class ZaloUserError(ZaloAPIException):
	"""Thrown by ``zaloapi`` when wrong values are entered."""
	def __init__(self, message=None):
		self.message = message
		super().__init__(message)


class EncodePayloadError(ZaloAPIException):
	"""Raised by ``zaloapi`` if:

    - The secret key is not correct to encode the payload
    - Payload data does not match.
    - A conflict occurred when encoding the payload.
    """
	def __init__(self, message=None):
		self.message = message
		super().__init__(message)
		
		
class DecodePayloadError(ZaloAPIException):
	"""Raised by ``zaloapi`` if:

    - The secret key is not correct to decode the payload
    - Payload data does not match.
    - A conflict occurred when decoding the payload.
    """
	def __init__(self, message=None):
		self.message = message
		super().__init__(message)
		



def Parse(text, styles=None, parse_mode=None):
	styles = styles or []
	if parse_mode == "Markdown":
		new_text, parse_list = parse_markdown(text)
	else:
		new_text, parse_list = markdown_message(text)
	
	return new_text, parse_list


def parse_markdown(text):
	"""Parses Markdown text and returns a list of Markdown elements.

	Args:
		text (str): The Markdown text to parse.

	Returns:
		list: A list of dictionaries, each representing a Markdown element.
		Each dictionary has the following keys:
			- 'start': The starting index of the element in the text.
			- 'end': The ending index of the element in the text.
			- 'length': The length of the element in the text.
			- 'text': The text content of the element.
			- 'char_len': The length of the Markdown character used for the element.
			- 'type': The type of Markdown element (e.g., 'bold', 'italic').
	"""

	markdown_elements = []
	markdown = {
		"**": "bold",
		"__": "underline",
		"_": "italic",
		"~~": "strike"
	}

	temp_text = text
	while any(temp_text.count(char) >= 2 for char in markdown.keys()):
		
		markup_start_positions = {
			char: temp_text.find(char)
			for char in markdown.keys()
		}

		
		sorted_markup_start_positions = dict(sorted(markup_start_positions.items(), key=lambda item: item[1]))
		
		for char, start in sorted_markup_start_positions.items():
			if start < 0:
				continue

			end = temp_text.rfind(char, start + len(char))
			
			if temp_text[start + len(char):end].count(char) >= 2:
				end = temp_text.find(char, start + len(char))

			if end < 0:
				continue
			
			element = {
				"start": start,
				"end": end,
				"length": end - start - len(char) + 1,
				"text": temp_text[start + len(char):end],
				"char_len": len(char),
				"type": markdown[char]
			}
			markdown_elements.append(element)
			
			temp_text = temp_text[:start] + temp_text[start + len(char):end] + temp_text[end + len(char):]
			break
	
	markdown_elements = sorted(markdown_elements, key=lambda x: x['start'])
	
	for element in markdown_elements:
		text = text[:element["start"]] + text[element["start"] + element["char_len"]:element["end"]] + text[element["end"] + element["char_len"]:]
		element["start"] -= 1
		element["end"] += 2
	
	return text, markdown_elements


def markdown_message(text):
	except_list = []
	markdown_list = []
	markdown_chars = {
		"<b>": "bold",
		"<i>": "italic",
		"<u>": "underline",
		"<s>": "strike",
	}
	for char, name in markdown_chars.items():
		start = 0
		while True:
			start = text.find(char, start)
			if start == -1:
				break
			
			end = text.find("</" + char[1:], start)
			if end == -1:
				break
			
			length = len(text[start + len(char):end])
			text = text.replace(char, "", 1)
			text = text.replace("</"+char[1:], "", 1)
			
			for charex in markdown_chars:
				if charex in text[start:end]:
					startex = text[start:end].find(charex)
					endex = text[start:end].find("</"+charex[1:], startex)
					if endex == -1:
						endex = text[end:].find("</"+charex[1:])
						if endex == -1:
							continue
						else:
							length -= 3
					else:
						length -= 7
					
					# except_list.append({"name": charex, "start": startex, "end": endex})
			
			markdown_list.append({
				"start": start - 1,
				"end": end + len(name),
				"length": length,
				"type": name,
				# "text": markdown_text
			})
			start = 0
	return text, markdown_list


def parse_html(text):
	"""Parses Markdown text and returns a list of Markdown elements.

	Args:
		text (str): The Markdown text to parse.

	Returns:
		list: A list of dictionaries, each representing a Markdown element.
		Each dictionary has the following keys:
			- 'start': The starting index of the element in the text.
			- 'end': The ending index of the element in the text.
			- 'length': The length of the element in the text.
			- 'text': The text content of the element.
			- 'char_len': The length of the Markdown character used for the element.
			- 'type': The type of Markdown element (e.g., 'bold', 'italic').
	"""

	markdown_elements = []
	markdown = {
		"<b>": "bold",
		"<u>": "underline",
		"<i>": "italic",
		"<s>": "strike"
	}

	temp_text = text
	while any(temp_text.count(char) >= 1 and temp_text.count("</" + char[1:]) >= 1 for char in markdown.keys()):
		
		markup_start_positions = {
			char: temp_text.find(char)
			for char in markdown.keys()
		}

		
		sorted_markup_start_positions = dict(sorted(markup_start_positions.items(), key=lambda item: item[1]))
		
		for char, start in sorted_markup_start_positions.items():
			if start < 0:
				continue

			end = temp_text.rfind("</" + char[1:], start + len(char))
			
			if temp_text[start + len(char):end].count("</" + char[1:]) >= 1:
				end = temp_text.find("</" + char[1:], start + len(char))

			if end < 0:
				continue
			
			element = {
				"start": start,
				"end": end,
				"length": end - start - len(char),
				"text": temp_text[start + len(char):end],
				"char_len": len(char),
				"type": markdown[char]
			}
			markdown_elements.append(element)
			
			temp_text = temp_text[:start] + temp_text[start + len(char):end] + temp_text[end + len(char) + 1:]
			break
	
	markdown_elements = sorted(markdown_elements, key=lambda x: x['start'])
	
	for element in markdown_elements:
		text = text[:element["start"]] + text[element["start"] + element["char_len"]:element["end"]] + text[element["end"] + element["char_len"] + 1:]
		element["start"] -= 1
	
	return text, markdown_elements


# -*- coding: UTF-8 -*-
import attr
import random
import asyncio
import aiohttp

from .. import _util, _exception

class State(object):
	def __init__(cls):
		cls._config = {}
		cls._headers = _util.HEADERS
		cls._cookies = _util.COOKIES
		cls.user_id = None
		cls.user_imei = None
		cls._loggedin = False
	
	def is_logged_in(cls):
		return cls._loggedin
	
	def set_cookies(cls, cookies):
		cls._cookies = cookies
	
	def set_secret_key(cls, secret_key):
		cls._config["secret_key"] = secret_key
	
	async def get_cookies(cls):
		return cls._cookies
	
	async def get_secret_key(cls):
		return cls._config.get("secret_key")
	
	async def _get(cls, *args, **kwargs):
		async with aiohttp.ClientSession() as session:
			async with session.get(*args, **kwargs, headers=cls._headers, cookies=cls._cookies) as response:
				return await response.json(content_type=None)
		
	async def _post(cls, *args, **kwargs):
		async with aiohttp.ClientSession() as session:
			async with session.post(*args, **kwargs, headers=cls._headers, cookies=cls._cookies) as response:
				return await response.json(content_type=None)
	
	async def login(cls, phone, password, imei, session_cookies=None, user_agent=None):
		if cls._cookies and cls._config.get("secret_key"):
			cls._loggedin = True
			return
			
		if user_agent:
			cls._headers["User-Agent"] = user_agent
			
		if cls._cookies:
			params = {
				"zpw_ver": 647,
				"type": 30,
				"imei": imei,
				"computer_name": "Web",
				"ts": _util.now(),
				"nretry": 0
			}
			try:
				data = await cls._get("https://wpa.chat.zalo.me/api/login/getLoginInfo", params=params)
				
				if data.get("error_code") == 0:
					cls._config = data.get("data")
					
					if cls._config.get("zpw_enk"):
						cls.user_imei = imei
						cls.user_id = cls._config.get("send2me_id")
						cls._config["secret_key"] = cls._config.get("zpw_enk")
						
					else:
						cls._loggedin = False
						raise _exception.ZaloLoginError("Unable to get `secret key`.")
						
				else:
					error = data.get("error_code")
					content = data.get("error_message")
					raise _exception.ZaloLoginError(f"Error #{error} when logging in: {content}")
			
			except _exception.ZaloLoginError as e:
				raise _exception.ZaloLoginError(str(e))
				
			except Exception as e:
				raise _exception.ZaloLoginError(f"An error occurred while logging in! {str(e)}")
		
		else:
			raise _exception.LoginMethodNotSupport("Login method is not supported yet")
		
	



import json

from . import _util
from ._core import Enum
from ._parse_message import Parse

class Message:
	def __init__(self, text=None, style=None, mention=None, parse_mode=None):
		self.text = text
		self.style = str(style) if style else None
		self.mention = str(mention) if mention else None
		self.parse_mode = str(parse_mode) if parse_mode else None
		
		
		if parse_mode:
			styles = json.loads(self.style)["styles"] if self.style else []
			if self.parse_mode == "Markdown":
				
				self.text, self.parse_list = Parse(self.text, self.style, "Markdown")
				
			elif self.parse_mode == "HTML":
				
				self.text, self.parse_list = Parse(self.text, self.style, "HTML")
			
			else:
				raise ValueError("Invalid Parse Mode, Only Support `Markdown` & `HTML`")
			
			if len(self.parse_list) == 1 and not styles:
			
				element = self.parse_list[0]
				self.style = MessageStyle(
					offset=element["start"],
					length=element["length"],
					style=element["type"]
				)
		
			elif len(self.parse_list) < 1:
				pass
		
			else:
				
				for element in self.parse_list:
					style = MessageStyle(
						offset=element["start"],
						length=element["length"],
						style=element["type"],
						auto_format=False
					)
					styles.append(style)
			
				self.style = MultiMsgStyle(styles)
			
			self.style = str(self.style) if self.style else None
	
				
	
	def __repr__(self):
		return f"Message(text={self.text!r}, style={self.style!r}, mention={self.mention!r}, parse_mode={self.parse_mode!r})"


class MessageStyle:
	def __new__(self, offset=0, length=1, style="font", color="ffffff", size="18", auto_format=True):
		self.offset = offset
		self.length = length
		self.style = style
		if type(offset) != int and type(length) != int:
			raise ValueError("Invalid Length, Offset! Length and Offset must be integers")
		
		if style == "bold":
			self.style = "b"
		elif style == "italic":
			self.style = "i"
		elif style == "underline":
			self.style = "u"
		elif style == "strike":
			self.style = "s"
		elif style == "color":
			self.style = "c_" + str(color).replace("#", "")
		elif style == "font":
			self.style = "f_" + str(size)
		else:
			self.style = "f_18"
		
		if auto_format:
			self.styleFormat = json.dumps({
				"styles": [{
					"start": self.offset,
					"len": self.length,
					"st": self.style
				}],
				"ver": 0
			})
		else:
			self.styleFormat = {
				"start": self.offset,
				"len": self.length,
				"st": self.style
			}
		
		return self.styleFormat
		

class MultiMsgStyle:
	def __init__(self, listStyle):
		styles = []
		for style in listStyle:
			styles.append(style)
			
		self.styleFormat = json.dumps({
			"styles": styles,
				"ver": 0
		})
	
	def __str__(self):
		return self.styleFormat


class MessageReaction:
	def __new__(self, messageObject, auto_format=True):
		self.msgId = messageObject.msgId
		self.cliMsgId = messageObject.cliMsgId
		self.msgType = _util.getClientMessageType(messageObject.msgType)
		if not isinstance(msgType, int):
			raise ValueError("Msg Type must be int")
		
		if auto_format:
			self.reactionFormat = [{
				"gMsgID": int(self.msgId),
				"cMsgID": int(self.cliMsgId),
				"msgType": int(self.msgType)
			}]
			
		else:
			self.reactionFormat = {
				"gMsgID": int(self.msgId),
				"cMsgID": int(self.cliMsgId),
				"msgType": int(self.msgType)
			}
		
		return self.reactionFormat


class Mention:
	def __new__(self, uid, length=1, offset=0, auto_format=True):
		self.user_id = uid
		self.offset = offset
		self.length = length
		self.type = 1 if uid == "-1" else 0
		if type(offset) != int and type(length) != int:
			raise ValueError("Invalid Length, Offset! Length and Offset must be integers")
		
		if auto_format:
			self.mentionFormat = json.dumps([{
				"pos": self.offset,
				"len": self.length,
				"uid": self.user_id,
				"type": self.type
			}])
		else:
			self.mentionFormat = {
				"pos": self.offset,
				"len": self.length,
				"uid": self.user_id,
				"type": self.type
			}
			
		return self.mentionFormat


class MultiMention:
	def __init__(self, listMention):
		mentions = []
		for mention in listMention:
			mentions.append(mention)
			
		self.mentionFormat = json.dumps(mentions)
	
	def __str__(self):
		return self.mentionFormat


from munch import DefaultMunch
from dataclasses import dataclass

class User(DefaultMunch):
	def __repr__(self):
		attrs = [f"{key}={value!r}" for key, value in self.__dict__.items()]
		return f"User({', '.join(attrs)})"


class Group(DefaultMunch):
	def __repr__(self):
		attrs = [f"{key}={value!r}" for key, value in self.__dict__.items()]
		return f"Group({', '.join(attrs)})"


class ContextObject(DefaultMunch):
	def __repr__(self):
		attrs = [f"{key}={value!r}" for key, value in self.__dict__.items()]
		return f"Context({', '.join(attrs)})"


class MessageObject(DefaultMunch):
	def __repr__(self):
		attrs = [f"{key}={value!r}" for key, value in self.__dict__.items()]
		return f"Message({', '.join(attrs)})"


class EventObject(DefaultMunch):
	def __repr__(self):
		attrs = [f"{key}={value!r}" for key, value in self.__dict__.items()]
		return f"GroupEvent({', '.join(attrs)})"


from setuptools import setup, find_packages

setup(
    name="zl2x",
    version="1.0.2",
    description="Zalo API for Python",
    packages=find_packages(),
    install_requires=[
        'requests',
        'aiohttp',
        'aenum',
        'attr',
        'pycryptodome',
        'datetime',
        'munch',
        'websockets'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ]
)